from datetime import datetime

QR_CODE_GENERATION_VERSION_DATE = datetime(year=2020, month=9, day=8, hour=12)
SIZE_DICT = {'t': 6, 's': 12, 'm': 18, 'l': 30, 'h': 48}
ERROR_CORRECTION_DICT = {'L': 'l', 'M': 'm', 'Q': 'q', 'H': 'h'}
DEFAULT_MODULE_SIZE = 'm'
DEFAULT_BORDER_SIZE = 4
DEFAULT_VERSION = None
DEFAULT_IMAGE_FORMAT = 'svg'
DEFAULT_ERROR_CORRECTION = 'm'
DEFAULT_CACHE_ENABLED = True
DEFAULT_URL_SIGNATURE_ENABLED = True

ALLOWS_EXTERNAL_REQUESTS_FOR_REGISTERED_USER = 'ALLOWS_EXTERNAL_REQUESTS_FOR_REGISTERED_USER'
SIGNING_KEY = 'SIGNING_KEY'
TOKEN_LENGTH = 'TOKEN_LENGTH'
SIGNING_SALT = 'SIGNING_SALT'
